# YouCompleteMe (https://github.com/ycm-core/YouCompleteMe) configuration file
# that enables it to find the LLVM and Python includes from a conda
# environment, independent of the Conda environment location and Python and
# LLVM versions.

import os
import sys

from pathlib import Path

CONDA_PREFIX = os.environ['CONDA_PREFIX']
LLVMLITE_DIR = Path(__file__).parent
VER = sys.version_info
PYTHON_INCLUDE_NAME = f'python{VER.major}.{VER.minor}'

CONDA_INCLUDE_DIR = Path(CONDA_PREFIX, 'include')
PYTHON_INCLUDE_DIR = Path(CONDA_INCLUDE_DIR, PYTHON_INCLUDE_NAME)
FFI_DIR = Path(LLVMLITE_DIR, 'ffi')

flags = [
    # Include dirs
    f'-I{CONDA_INCLUDE_DIR}',
    f'-I{PYTHON_INCLUDE_DIR}',
    f'-I{FFI_DIR}',
    # C++ standard to which LLVM 14 is developed
    '-std=c++14',
    # Force language to C++ so that core.h is treated as C++
    '-x', 'c++',
]


# This function is called by YCM to obtain the config from this file.
def Settings(**kwargs):
    return {'flags': flags}


# Executing this configuration file standalone outside of YCM prints the
# settings to aid with debugging the configuration.
if __name__ == '__main__':
    print(Settings())



import lief
import pathlib

for path in pathlib.Path(".").rglob("**/*.dll"):
    print("path", path)
    dll = lief.PE.parse(str(path))

    if path.name.lower().startswith("msvcp140"):
        continue

    if hasattr(dll, "delay_imports"):
        delay_imports = {x.name for x in dll.delay_imports}
        print("Delay imports:", delay_imports)
        expected_delay_imports = {"SHELL32.dll", "ole32.dll"}
        assert (
            delay_imports == expected_delay_imports
        ), f"Unexpected delay imports: {delay_imports}"

    imports = {x.name for x in dll.imports}
    print("Regular imports:", imports)
    # Normalize delvewheel-mangled MSVCP import (e.g. msvcp140-<hash>.dll)
    imports = {("MSVCP140.dll" if name.lower().startswith("msvcp140") else name) for name in imports}
    expected_imports = {
        "ADVAPI32.dll",
        "KERNEL32.dll",
        "MSVCP140.dll",
        "VCRUNTIME140.dll",
        "VCRUNTIME140_1.dll",
        "api-ms-win-crt-convert-l1-1-0.dll",
        "api-ms-win-crt-environment-l1-1-0.dll",
        "api-ms-win-crt-heap-l1-1-0.dll",
        "api-ms-win-crt-locale-l1-1-0.dll",
        "api-ms-win-crt-math-l1-1-0.dll",
        "api-ms-win-crt-runtime-l1-1-0.dll",
        "api-ms-win-crt-stdio-l1-1-0.dll",
        "api-ms-win-crt-string-l1-1-0.dll",
        "api-ms-win-crt-time-l1-1-0.dll",
        "api-ms-win-crt-utility-l1-1-0.dll",
        "ntdll.dll",
    }
    assert imports == expected_imports, (
        f"Unexpected imports: {imports - expected_imports}\n"
        f"Missing imports: {expected_imports - imports}"
    )



#!/usr/bin/env python

import json
import os
from pathlib import Path


event = os.environ.get("GITHUB_EVENT_NAME")
label = os.environ.get("GITHUB_LABEL_NAME")
inputs = os.environ.get("GITHUB_WORKFLOW_INPUT", "{}")

runner_mapping = {
    "linux-64": "ubuntu-24.04",
    "linux-aarch64": "ubuntu-24.04-arm",
    "osx-arm64": "macos-14",
    "win-64": "windows-2025",
}

default_include = [
    # linux-64
    {
        "runner": runner_mapping["linux-64"],
        "platform": "linux-64",
        "recipe": "llvmdev",
    },
    {
        "runner": runner_mapping["linux-64"],
        "platform": "linux-64",
        "recipe": "llvmdev_for_wheel",
    },
    # linux-aarch64
    {
        "runner": runner_mapping["linux-aarch64"],
        "platform": "linux-aarch64",
        "recipe": "llvmdev",
    },
    {
        "runner": runner_mapping["linux-aarch64"],
        "platform": "linux-aarch64",
        "recipe": "llvmdev_for_wheel",
    },
    # osx-arm64
    {
        "runner": runner_mapping["osx-arm64"],
        "platform": "osx-arm64",
        "recipe": "llvmdev",
    },
    {
        "runner": runner_mapping["osx-arm64"],
        "platform": "osx-arm64",
        "recipe": "llvmdev_for_wheel",
    },
    # win-64
    {
        "runner": runner_mapping["win-64"],
        "platform": "win-64",
        "recipe": "llvmdev",
    },
    {
        "runner": runner_mapping["win-64"],
        "platform": "win-64",
        "recipe": "llvmdev_for_wheel",
    },
]

print(
    "Deciding what to do based on event: "
    f"'{event}', label: '{label}', inputs: '{inputs}'"
)
if event == "pull_request":
    print("pull_request detected")
    include = default_include
elif event == "label" and label == "build_on_gha":
    print("build label detected")
    include = default_include
elif event == "workflow_dispatch":
    print("workflow_dispatch detected")
    params = json.loads(inputs)
    platform = params.get("platform", "all")
    recipe = params.get("recipe", "all")

    # Start with the full matrix
    filtered_matrix = default_include

    # Filter by platform if a specific one is chosen
    if platform != "all":
        filtered_matrix = [
            item for item in filtered_matrix if item["platform"] == platform
        ]

    # Filter by recipe if a specific one is chosen
    if recipe != "all":
        filtered_matrix = [
            item for item in filtered_matrix if item["recipe"] == recipe
        ]

    include = filtered_matrix
else:
    include = {}

matrix = {"include": include}
print(f"Emitting matrix:\n {json.dumps(matrix, indent=4)}")

Path(os.environ["GITHUB_OUTPUT"]).write_text(f"matrix={json.dumps(matrix)}")
